@@ -35,6 +35,7 @@ import keyboard
 from itertools import cycle
 from functools import reduce
 import pytz
+from collections import Counter
 
 variables = {}
 functions = {}
@@ -87,11 +88,10 @@ def unlockKeywordArguments(**kwargs):
 for arg in args:
     Argument = arg
 
-def write(text):
-    print(f"{text}")
-
-def writenoend(text):
-    print(f"{text}", end="")
+def write(text, end="\n", flush=False, seperate=""):
+    text = str(text)
+    modifiedtext = text.replace(" ", seperate)
+    print(modifiedtext, end=end, flush=flush)
 
 def showmachinefulldata():
     return f"""{platform.platform()}
@@ -1210,6 +1210,30 @@ def varmultiplymultiply(variable):
 def vardividedivide(variable):
     variables[variable] /= 2
 
+def varbxorbxor(variable):
+    variables[variable] ^= 2
+
+def varfloordivfloordiv(var):
+    variables[var] //= 2
+
+def varpowerpower(var):
+    variables[var] **= 2
+
+def varmodulomodulo(var):
+    variables[var] %= 2
+
+def varbandband(var):
+    variables[var] &= 2
+
+def varborbor(var):
+    variables[var] |= 2
+
+def varshiftleft(var):
+    variables[var] <<= 1
+
+def varshiftright(var):
+    variables[var] >>= 1
+
 def addto(var, amount):
     variables[var] += amount
 
@@ -1228,6 +1252,36 @@ def divideto(var, amount):
     else:
         variables[var] /= amount
 
+def bxorto(var, amount):
+    variables[var] ^= amount
+
+def floordivideto(var, amount):
+    if amount == 0:
+        if variables[var] == 0:
+            variables[var] = float("nan")
+        else:
+            variables[var] = float("inf")
+    else:
+        variables[var] //= amount
+
+def powerto(var, amount):
+    variables[var] **= amount
+
+def moduloto(var, amount):
+    variables[var] %= amount
+
+def bitwiseandto(var, amount):
+    variables[var] &= amount
+
+def bitwiseorto(var, amount):
+    variables[var] |= amount
+
+def shiftleftto(var, amount):
+    variables[var] <<= amount
+
+def shiftrightto(var, amount):
+    variables[var] >>= amount
+
 def If(condition, action):
     if eval(condition, {}, variables):
         exec(action)
@@ -1304,11 +1358,16 @@ def Class(name, body):
     classes[name] = body
 
 def newobject(name, classname):
-    if classname in classes:
-        objects[name] = {"__class__": classname}
-        exec(classes[classname], {}, {"self": objects[name], "write": write})
-    else:
+    if classname not in classes:
         print(f"Class {classname} was not found.")
+        return None
+    obj = {}
+    obj["__class__"] = classname
+    objects[name] = obj
+
+    ns = {"self": obj}
+    exec(classes[classname], ns, ns)
+    return obj
 
 def contains(textct, text, custom_action=None, custom_action2=None):
     if textct in text:
@@ -1456,7 +1515,7 @@ def List(listvariable):
 def getvalue(dictionary, key):
     return dictionary.get(key)
 
-def popvalue(d, key):
+def popvalue(dictionary, key):
     return dictionary.pop(key, None)
 
 def listkeys(dictionary):
@@ -2274,8 +2333,8 @@ def primeorcomposite(num):
 def getid(variable):
     return id(variable)
 
-def tocenter(variable):
-    return variable.center()
+def tocenter(variable, width, fillchar=None):
+    return variable.center(width, fillchar=fillchar if fillchar else "")
 
 def initstylesandformats():
     init(autoreset=False)
@@ -2468,9 +2527,12 @@ def getstats(getfilestatsvarname, toget):
     return stats.st_toget
 
 def createbranch(appvar, funcname, code, branch="/"):
-    fullcode = f"""@appvar.get({branch}, response_class=HTMLResponse)
-    def {funcname}():
-        {code}"""
+    indentedcode = "\n    ".join(code.splitlines())
+    fullcode = (
+        f"@{appvar}.get('{branch}', response_class=HTMLResponse)\n"
+        f"def {funcname}():\n"
+        f"    {indentedcode}"
+    )
     exec(fullcode, globals())
     return fullcode
 
@@ -2978,3 +3040,365 @@ def center(efn_root):
 
 def noneanchor(efn_root):
     return tk.NONE
+
+def leftjustify(text, width, fillchar=None):
+    text.ljust(width, fillchar=fillchar if fillchar else "")
+
+def rightjustify(text, width, fillchar=None):
+    text.rjust(width, fillchar=fillchar if fillchar else "")
+
+def staticmethod(funcname, code):
+    indentedcode = "\n    ".join(code.splitlines())
+    fullcode = f"@staticmethod\ndef {funcname}():\n    {indentedcode}"
+    return fullcode
+
+def property(funcname, code):
+    indentedcode = "\n    ".join(code.splitlines())
+    fullcode = f"@property\ndef {funcname}():\n    {indentedcode}"
+    return fullcode
+
+def topgui(efn_root):
+    return tk.TOP
+
+def bottomgui(efn_root):
+    return tk.BOTTOM
+
+def boundbox(efn_root, widget, tobbox):
+    return widget.bbox(f"{tobbox}")
+
+def configurescrollregion(efn_root, widget, scrollregion):
+    widget.configure(scrollregion=scrollregion)
+
+def rightgui(efn_root):
+    return tk.RIGHT
+
+def leftgui(efn_root):
+    return tk.LEFT
+
+def xview(efn_root, widget):
+    return widget.xview
+
+def yview(efn_root, widget):
+    return widget.yview
+
+def configurefill(efn_root, widget, fill):
+    widget.configure(fill=fill)
+
+def setviewscrollbar(efn_root, scrollbarvariable):
+    return scrollbarvariable.set
+
+def configurexscrollcommand(efn_root, widget, xscrollcommand):
+    widget.configure(widget, xscrollcommand=xscrollcommand)
+
+def configureyscrollcommand(efn_root, widget, yscrollcommand):
+    widget.configure(widget, yscrollcommand=yscrollcommand)
+
+def configuredash(efn_root, canvas, dash):
+    canvas.configure(dash=dash)
+
+def configureoffset(efn_root, canvas, offset):
+    canvas.configure(offset=offset)
+
+def configuredisableddash(efn_root, canvas, disableddash):
+    canvas.configure(disableddash=disableddash)
+
+def createwindow(efn_root, canvas, x, y, parent, anchor, width, height, tags, state):
+    canvaswindow = canvas.create_window(x, y, window=parent, anchor=anchor, width=width, height=height, tags=tags, state=state)
+    return canvaswindow
+
+def callthis(tocall, subfunction=None, *args, **kwargs):
+    if subfunction:
+        tocall[subfunction](*args, **kwargs)
+    else:
+        tocall(*args, **kwargs)
+
+def addxscroll(efn_root, widget, orient="horizontal"):
+    parent = widget.master or efn_root
+    xscroll = tk.Scrollbar(parent, orient=orient)
+    widget.configure(xscrollcommand=xscroll.set)
+    xscroll.configure(command=widget.xview)
+    xscroll.pack(side="bottom", fill="x")
+    return xscroll
+
+def addyscroll(efn_root, widget, orient="horizontal"):
+    parent = widget.master or efn_root
+    yscroll = tk.Scrollbar(parent, orient=orient)
+    widget.configure(yscrollcommand=yscroll.set)
+    yscroll.configure(command=widget.yview)
+    yscroll.pack(side="bottom", fill="y")
+    return yscroll
+
+
+class LoadingWidget(tk.Canvas):
+    def __init__(self, efn_root, efnframe=None, size=100, line_width=10, speed=5):
+        parent = efnframe if efnframe else efn_root
+        flag = "_loading_widget_created"
+
+        if hasattr(parent, flag):
+            return
+
+        super().__init__(parent, width=size, height=size, bg='white', highlightthickness=0)
+        self.size = size
+        self.line_width = line_width
+        self.speed = speed
+        self.angle = 0
+        self.draw_arc()
+        self.animate()
+        self.pack()
+        setattr(parent, flag, True)
+
+    def draw_arc(self):
+        self.delete("arc")
+        self.create_arc(
+            self.line_width,
+            self.line_width,
+            self.size - self.line_width,
+            self.size - self.line_width,
+            start=self.angle,
+            extent=90,
+            style='arc',
+            outline='green',
+            width=self.line_width,
+            tags="arc"
+        )
+
+    def animate(self):
+        self.angle = (self.angle + self.speed) % 360
+        self.draw_arc()
+        self.after(50, self.animate)
+
+def greenwhiteload(efn_root, efnframe=None, width=200, height=50):
+    parent = efnframe if efnframe else efn_root
+    flag = "__greenwhiteload_widget_created"
+
+    if hasattr(parent, flag):
+        return None, None
+
+    mainframe = tk.Frame(parent, width=width, height=height)
+    mainframe.pack()
+
+    greenlbl = tk.Label(mainframe, width=width, height=height, bg='green')
+    greenlbl.pack()
+
+    whitelbl = tk.Label(mainframe, width=width//4, height=height, bg='white')
+    whitelbl.place(x=0, y=0)
+
+    step = 5
+    pos = 0
+
+    def move():
+        nonlocal pos
+        pos += step
+        if pos > width:
+            pos = 0
+        whitelbl.place(x=pos, y=0)
+        whitelbl.after(50, move)
+
+    move()
+    setattr(parent, flag, True)
+    return mainframe, whitelbl
+
+def accessnonlocal(variable):
+    def inner():
+        nonlocal variable
+    return inner()
+
+def customizabletimezonewidget(efn_root, timezonesdict, efnframe=None, bg=None, fg=None):
+    parent = efnframe if efnframe else efn_root
+    flag = f"timezone_widget_created_with_{timezonesdict.get()}"
+    cities = timezonesdict.get()
+
+    if not hasattr(parent, flag):
+        mainframe = tk.Frame(parent, bg=bg if bg else "white")
+        mainframe.pack()
+
+        labels = {}
+        for city in cities:
+            label = tk.Label(mainframe, font=('Arial', 14), bg=bg if bg else "white", fg=fg if fg else "black")
+            label.pack(anchor='w')
+            labels[city] = label
+
+        def update_time():
+            for city, zone in cities.items():
+                tz = pytz.timezone(zone)
+                time = datetime.now(tz).strftime('%H:%M:%S')
+                labels[city].config(text=f"{city}: {time}")
+            mainframe.after(1000, update_time)
+
+        update_time()
+        setattr(parent, flag, True)
+        return mainframe, labels
+
+    return None, None
+
+class ComplexSwitch(tk.Frame):
+    def __init__(self, efn_root, efnframe=None, text='Switch', command=None, **kwargs):        
+        parent = efn_root if not efnframe else efnframe
+        super().__init__(parent, **kwargs)
+        
+        self._command = command
+        self._state = False
+
+        self.main_label = tk.Label(self, text=text, width=10, fg='white',
+                                   font=('Arial', 12, 'bold'))
+        self.main_label.pack(side='left', padx=(0,5), pady=5)
+        
+        self.sub_label = tk.Label(self, text='OFF', bg='white', fg='black',
+                                  width=5, font=('Arial', 10))
+        self.sub_label.pack(side='left', pady=5)
+
+        self.bind_widgets(self.main_label)
+        self.bind_widgets(self.sub_label)
+        self.bind_widgets(self)
+
+        self._update_appearance()
+
+    def bindwidgets(self, widget):
+        widget.bind('<Button-1>', self._on_click)
+
+    def onclick(self, event):
+        self._state = not self._state
+        self._update_appearance()
+        if self._command:
+            self._command(self._state)
+
+    def updateappearance(self):
+        if self._state:
+            self.main_label.config(bg='green')
+            self.sub_label.config(text='ON')
+        else:
+            self.main_label.config(bg='gray')
+            self.sub_label.config(text='OFF')
+
+def newline():
+    return "\n"
+
+def isanagram(wordone, wordtwo):
+    counterwordone = Counter(wordone)
+    counterwordtwo = Counter(wordtwo)
+    return counterwordone == counterwordtwo
+
+def counter(data):
+    Counter(data)
+
+def carriagereturn():
+    return f"\r"
+
+def carriagereturnbefore(times, line):
+    return '\r' * times + line
+
+def carriagereturnafter(times, line):
+    return line + '\r' * times
+
+def tab():
+    return f"\t"
+
+def tabbefore(times, text):
+    return '\t' * times + text
+
+def tabafter(times, text):
+    return text + '\t' * times
+
+def backspace():
+    return f"\b"
+
+def backspacebefore(times, text):
+    return '\b' * times + text
+
+def backspaceafter(times, text):
+    return text + '\b' * times
+
+def formfeed():
+    return f"\f"
+
+def formfeedbefore(times, text):
+    return '\f' * times + text
+
+def formfeedafter(times, text):
+    return text + '\f' * times
+
+def verticaltab():
+    return f"\v"
+
+def verticaltabbefore(times, text):
+    return '\v' * times + text
+
+def verticaltabafter(times, text):
+    return text + '\v' * times
+
+def octalline(code):
+    return f"\{code}"
+
+def octallinebefore(times, code, text):
+    return f"\{code}" * times + text
+
+def octallineafter(times, code, text):
+    return text + f"\{code}" * times
+
+def hexa():
+    return fr"\x"
+
+def hexabefore(times, code, text):
+    return fr"\x{code}" * times + text
+
+def hexaafter(times, code, text):
+    return text + fr"\x{code}" * times
+
+def unicode16():
+    return fr"\u"
+
+def unicode16before(times, code, text):
+    return fr"\u{code}" * times + text
+
+def unicode16after(times, code, text):
+    return text + fr"\u{code}" * times
+
+def unicode32():
+    return fr"\U"
+
+def unicode32before(times, code, text):
+    return fr"\U{code}" * times + text
+
+def unicode32after(times, code, text):
+    return text + fr"\U{code}" * times
+
+def createdictionary(data):
+    return {data}
+
+def createlistarray(data):
+    return [data]
+
+def pathorfileexists(pathorfile):
+    return os.path.exists(pathorfile)
+
+def isfile(fileorpath):
+    return os.path.isfile(fileorpath)
+
+def isdirectory(pathorfile):
+    return os.path.isdir(pathorfile)
+
+def isfibonacci(number):
+    return number >= 0 and (
+        (lambda x: int(math.isqrt(x))**2 == x)(5*number*number + 4) or
+        (lambda x: int(math.isqrt(x))**2 == x)(5*number*number - 4)
+    )
+
+def dataclass(classname, code):
+    indentedcode = "\n    ".join(code.splitlines())
+    fullcode = f"@dataclass\nclass {classname}():\n    {indentedcode}"
+    return fullcode
+
+def widgetx(efn_root, widget):
+    return widget.x
+
+def widgety(efn_root, widget):
+    return widget.y
+
+def returnunicode(value):
+    return ord(value)
+
+def bytearrayof(value):
+    return bytearray(value)
+
+def unicodepoint(value):
+    return chr(value)
